/*
 * This file and its contents are supplied under the terms of the
 * Common Development and Distribution License ("CDDL"), version 1.0.
 * You may only use this file in accordance with the terms of version
 * 1.0 of the CDDL.
 *
 * A full copy of the text of the CDDL should have accompanied this
 * source.  A copy of the CDDL is also available via the Internet at
 * http://www.illumos.org/license/CDDL.
 */

/*
 * Copyright 2012 David Hoeppner.  All rights reserved.
 */

#ifndef	_INET_DCCP_DCCP_STACK_H
#define	_INET_DCCP_DCCP_STACK_H

#include <sys/netstack.h>
#include <sys/cpuvar.h>

#ifdef	__cplusplus
extern "C" {
#endif

typedef struct dccp_stat_counter_s {
	uint64_t	dccp_rst_unsent;
} dccp_stat_counter_t;

typedef struct {
	uint64_t		dccp_stats_cnt;
	dccp_stat_counter_t	dccp_sc_stats;
} dccp_stats_cpu_t;

#define	DCCP_STAT(dccps, x)	\
	((dccps)->dccps_sc[CPU->cpu_seqid]->dccp_sc_stats.x++)

/*
 * DCCP stack instances
 */
typedef struct dccp_stack {
	netstack_t		*dccps_netstack;	/* Common netstack */

	uint_t			dccps_bind_fanout_size;
	struct dccp_df_s	*dccps_bind_fanout;

	/* Ports */
#define	DCCP_NUM_EPRIV_PORTS	64
	int			dccps_num_epriv_ports;
	in_port_t		dccps_epriv_ports[DCCP_NUM_EPRIV_PORTS];
	kmutex_t		dccps_epriv_port_lock;

	uint_t			dccps_next_port_to_try;

	in_port_t		dccps_min_anonpriv_port;

	/* Reset rate control */
	int64_t			dccps_last_rst_intrvl;
	uint32_t		dccps_rst_cnt;

	/* Tunables table */
	struct mod_prop_info_s	*dccps_propinfo_tbl;

	ldi_ident_t		dccps_ldi_ident;

	/* Cpu stats counter */
	dccp_stats_cpu_t	**dccps_sc;
	int			dccps_sc_cnt;
} dccp_stack_t;

#ifdef	__cplusplus
}
#endif

#endif	/* _INET_DCCP_DCCP_STACK_H */
